@@ -17,6 +17,7 @@
 #include <OpenImageIO/platform.h>
 // #include <OpenImageIO/refcnt.h>
 #include <OpenImageIO/strutil.h>
+#include <OpenImageIO/sysutil.h>
 #include <OpenImageIO/ustring.h>
 
 #ifdef _WIN32
@@ -56,7 +57,7 @@ inline filesystem::path
 u8path(string_view name)
 {
 #ifdef _WIN32
-    return filesystem::path(Strutil::utf8_to_utf16(name));
+    return filesystem::path(Strutil::utf8_to_utf16wstring(name));
 #else
     return filesystem::path(name.begin(), name.end());
 #endif
@@ -66,7 +67,7 @@ inline filesystem::path
 u8path(const std::string& name)
 {
 #ifdef _WIN32
-    return filesystem::path(Strutil::utf8_to_utf16(name));
+    return filesystem::path(Strutil::utf8_to_utf16wstring(name));
 #else
     return filesystem::path(name);
 #endif
@@ -166,18 +167,6 @@ Filesystem::generic_filepath(string_view filepath) noexcept
 
 
 
-#if OIIO_VERSION_LESS(2, 4, 0)
-// For older versions, preserve this link-compatible entry point
-void
-Filesystem::searchpath_split(const std::string& searchpath,
-                             std::vector<std::string>& dirs, bool validonly)
-{
-    dirs = searchpath_split(searchpath, validonly);
-}
-#endif
-
-
-
 std::vector<std::string>
 Filesystem::searchpath_split(string_view searchpath, bool validonly)
 {
@@ -268,6 +257,41 @@ Filesystem::searchpath_find(const std::string& filename_utf8,
 
 
 
+std::string
+Filesystem::find_program(string_view program)
+{
+    const filesystem::path filename(u8path(program));
+    bool abs = filename.is_absolute();
+
+    // If it's an absolute path, we are only checking if it's executable
+    if (abs)
+        return Filesystem::is_executable(program) ? std::string(program)
+                                                  : std::string();
+
+    // If it's found without searching, and it's executable, return its
+    // absolute path.
+    if (Filesystem::is_executable(program))
+        return pathstr(filesystem::absolute(filename));
+
+    // Relative filename, not yet found -- try each $PATH directory in turn
+    for (auto&& d_utf8 : searchpath_split(OIIO::Sysutil::getenv("PATH"))) {
+        const filesystem::path d(u8path(d_utf8));
+        filesystem::path f = d / filename;
+        // Strutil::print("\tPath = {}\n", f);
+        auto p = pathstr(filesystem::absolute(f));
+        if (is_executable(p))
+            return p;
+#ifdef _WIN32
+        if (!Strutil::iends_with(p, ".exe")
+            && is_executable(Strutil::concat(p, ".exe")))
+            return Strutil::concat(p, ".exe");
+#endif
+    }
+    return std::string();
+}
+
+
+
 bool
 Filesystem::get_directory_entries(const std::string& dirname,
                                   std::vector<std::string>& filenames,
@@ -358,6 +382,30 @@ Filesystem::is_regular(string_view path) noexcept
 
 
 
+bool
+Filesystem::is_executable(string_view path) noexcept
+{
+    if (!is_regular(path))
+        return false;
+    error_code ec;
+    auto stat = filesystem::status(u8path(path), ec);
+    auto perm = stat.permissions();
+#ifdef USE_STD_FILESYSTEM
+    return (perm & filesystem::perms::owner_exec) != filesystem::perms::none
+           || (perm & filesystem::perms::group_exec) != filesystem::perms::none
+           || (perm & filesystem::perms::others_exec)
+                  != filesystem::perms::none;
+#else
+    return (perm & filesystem::perms::owner_exe) != filesystem::perms::no_perms
+           || (perm & filesystem::perms::group_exe)
+                  != filesystem::perms::no_perms
+           || (perm & filesystem::perms::others_exe)
+                  != filesystem::perms::no_perms;
+#endif
+}
+
+
+
 bool
 Filesystem::create_directory(string_view path, std::string& err)
 {
@@ -456,7 +504,7 @@ Filesystem::unique_path(string_view model)
     // to convert char* to wchar_t* because they do not know the encoding
     // See boost/filesystem/path.hpp
     // The only correct way to do this is to do the conversion ourselves
-    std::wstring modelStr    = Strutil::utf8_to_utf16(model);
+    std::wstring modelStr    = Strutil::utf8_to_utf16wstring(model);
 #    else
     std::string modelStr = model.str();
 #    endif
@@ -495,9 +543,8 @@ Filesystem::fopen(string_view path, string_view mode)
 {
 #ifdef _WIN32
     // on Windows fopen does not accept UTF-8 paths, so we convert to wide char
-    std::wstring wpath = Strutil::utf8_to_utf16(path);
-    std::wstring wmode = Strutil::utf8_to_utf16(mode);
-
+    std::wstring wpath = Strutil::utf8_to_utf16wstring(path);
+    std::wstring wmode = Strutil::utf8_to_utf16wstring(mode);
     return ::_wfopen(wpath.c_str(), wmode.c_str());
 #else
     // on Unix platforms passing in UTF-8 works
@@ -538,7 +585,7 @@ Filesystem::open(OIIO::ifstream& stream, string_view path,
 #ifdef _WIN32
     // Windows std::ifstream accepts non-standard wchar_t*
     // On MingW, we use our own OIIO::ifstream
-    std::wstring wpath = Strutil::utf8_to_utf16(path);
+    std::wstring wpath = Strutil::utf8_to_utf16wstring(path);
     stream.open(wpath.c_str(), mode);
     stream.seekg(0, std::ios_base::beg);  // force seek, otherwise broken
 #else
@@ -555,7 +602,7 @@ Filesystem::open(OIIO::ofstream& stream, string_view path,
 #ifdef _WIN32
     // Windows std::ofstream accepts non-standard wchar_t*
     // On MingW, we use our own OIIO::ofstream
-    std::wstring wpath = Strutil::utf8_to_utf16(path);
+    std::wstring wpath = Strutil::utf8_to_utf16wstring(path);
     stream.open(wpath.c_str(), mode);
 #else
     stream.open(path, mode);
@@ -570,7 +617,7 @@ Filesystem::open(string_view path, int flags)
 #ifdef _WIN32
     // on Windows _open does not accept UTF-8 paths, so we convert to wide
     // char and use _wopen.
-    std::wstring wpath = Strutil::utf8_to_utf16(path);
+    std::wstring wpath = Strutil::utf8_to_utf16wstring(path);
     return ::_wopen(wpath.c_str(), flags);
 #else
     // on Unix platforms passing in UTF-8 works
