@@ -46,6 +46,11 @@
 #    define STBI__ASAN OIIO_NO_SANITIZE_ADDRESS
 #endif
 #define stbsp__uintptr std::uintptr_t
+
+#ifdef OpenImageIO_SANITIZE
+#    define STB_SPRINTF_NOUNALIGNED
+#endif
+
 #include "stb_sprintf.h"
 
 
@@ -347,6 +352,7 @@ Strutil::escape_chars(string_view unescaped)
             case '\r': c = 'r'; break;
             case '\f': c = 'f'; break;
             case '\a': c = 'a'; break;
+            default: break;
             }
             s.insert(i, 1, c);
         }
@@ -374,10 +380,8 @@ Strutil::unescape_chars(string_view escaped)
                 case 'b': s[i] = '\b'; break;
                 case 'r': s[i] = '\r'; break;
                 case 'f': s[i] = '\f'; break;
-                case 'a':
-                    s[i] = '\a';
-                    break;
-                    // default case: the deletion is enough (backslash and quote)
+                case 'a': s[i] = '\a'; break;
+                default: break;  // the deletion is enough (backslash and quote)
                 }
             } else if (c >= '0' && c < '8') {
                 // up to 3 octal digits
@@ -776,16 +780,15 @@ Strutil::split(string_view str, std::vector<string_view>& result,
 std::string
 Strutil::concat(string_view s, string_view t)
 {
-    size_t sl  = s.size();
-    size_t tl  = t.size();
+    size_t sl = s.size();
+    size_t tl = t.size();
+    if (sl == 0)
+        return t;
+    if (tl == 0)
+        return s;
     size_t len = sl + tl;
-    std::unique_ptr<char[]> heap_buf;
-    char local_buf[256];
-    char* buf = local_buf;
-    if (len > sizeof(local_buf)) {
-        heap_buf.reset(new char[len]);
-        buf = heap_buf.get();
-    }
+    char* buf;
+    OIIO_ALLOCATE_STACK_OR_HEAP(buf, char, len);
     memcpy(buf, s.data(), sl);
     memcpy(buf + sl, t.data(), tl);
     return std::string(buf, len);
@@ -870,7 +873,7 @@ Strutil::replace(string_view str, string_view pattern, string_view replacement,
 // appropriate for error messages.
 
 std::wstring
-Strutil::utf8_to_utf16(string_view str) noexcept
+Strutil::utf8_to_utf16wstring(string_view str) noexcept
 {
     try {
         std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>, wchar_t> conv;
@@ -882,6 +885,17 @@ Strutil::utf8_to_utf16(string_view str) noexcept
 
 
 
+#if OPENIMAGEIO_VERSION < 30000
+// DEPRECATED(2.5) and slated for removal in 3.0.
+std::wstring
+Strutil::utf8_to_utf16(string_view str) noexcept
+{
+    return utf8_to_utf16wstring(str);
+}
+#endif
+
+
+
 std::string
 Strutil::utf16_to_utf8(const std::wstring& str) noexcept
 {
@@ -895,6 +909,19 @@ Strutil::utf16_to_utf8(const std::wstring& str) noexcept
 
 
 
+std::string
+Strutil::utf16_to_utf8(const std::u16string& str) noexcept
+{
+    try {
+        std::wstring_convert<std::codecvt_utf8_utf16<char16_t>, char16_t> conv;
+        return conv.to_bytes(str);
+    } catch (const std::exception&) {
+        return std::string();
+    }
+}
+
+
+
 char*
 Strutil::safe_strcpy(char* dst, string_view src, size_t size) noexcept
 {
@@ -1587,7 +1614,6 @@ Strutil::strtod(const char* nptr, char** endptr) noexcept
     // Can use strtod_l on platforms that support it
 #if defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__) \
     || defined(__FreeBSD_kernel__) || defined(__GLIBC__)
-    // static initialization inside function is thread-safe by C++11 rules!
     return strtod_l(nptr, endptr, c_loc);
 #elif defined(_WIN32)
     // Windows has _strtod_l
@@ -1667,7 +1693,7 @@ Strutil::stof(string_view s, size_t* pos)
     // will use the "short string optimization", meaning that this string
     // creation will NOT need an allocation/free for most strings we expect
     // to hold a text representation of a float.
-    return Strutil::stof(std::string(s).c_str(), pos);
+    return Strutil::stof(std::string(s), pos);
 }
 
 
