@@ -38,7 +38,7 @@ static std::string
 compute_sha1(Oiiotool& ot, ImageInput* input, int subimage)
 {
     SHA1 sha;
-    const ImageSpec& spec(input->spec());
+    ImageSpec spec = input->spec_dimensions(subimage);
     if (spec.deep) {
         // Special handling of deep data
         DeepData dd;
@@ -46,24 +46,25 @@ compute_sha1(Oiiotool& ot, ImageInput* input, int subimage)
             std::string err = input->geterror();
             if (err.empty())
                 err = "could not read image";
-            ot.errorf("-info", "SHA-1: %s", err);
+            ot.errorfmt("-info", "SHA-1: {}", err);
             return std::string();
         }
         // Hash both the sample counts and the data block
         sha.append(dd.all_samples());
         sha.append(dd.all_data());
     } else {
-        imagesize_t size = input->spec().image_bytes(true /*native*/);
+        imagesize_t size = spec.image_bytes(true /*native*/);
         if (size >= std::numeric_limits<size_t>::max()) {
-            ot.errorf("-info", "SHA-1: unable to compute, image is too big");
+            ot.error("-info", "SHA-1: unable to compute, image is too big");
             return std::string();
         } else if (size != 0) {
             std::unique_ptr<char[]> buf(new char[size]);
-            if (!input->read_image(TypeDesc::UNKNOWN /*native*/, &buf[0])) {
+            if (!input->read_image(subimage, 0, 0, spec.nchannels,
+                                   TypeDesc::UNKNOWN /*native*/, &buf[0])) {
                 std::string err = input->geterror();
                 if (err.empty())
                     err = "could not read image";
-                ot.errorf("-info", "SHA-1: %s", err);
+                ot.errorfmt("-info", "SHA-1: {}", err);
                 return std::string();
             }
             sha.append(&buf[0], size);
@@ -110,11 +111,12 @@ print_nums(std::ostream& out, int n, const T* val, string_view sep = " ",
 template<typename T>
 static bool
 dump_flat_data(std::ostream& out, ImageInput* input,
-               const print_info_options& opt)
+               const print_info_options& opt, int subimage)
 {
-    const ImageSpec& spec(input->spec());
+    ImageSpec spec = input->spec_dimensions(subimage);
     std::vector<T> buf(spec.image_pixels() * spec.nchannels);
-    if (!input->read_image(BaseTypeFromC<T>::value, &buf[0])) {
+    if (!input->read_image(subimage, 0, 0, spec.nchannels,
+                           BaseTypeFromC<T>::value, &buf[0])) {
         Strutil::print(out, "    dump data Error: could not read image: {}\n",
                        input->geterror());
         return false;
@@ -193,14 +195,15 @@ dump_flat_data(std::ostream& out, ImageInput* input,
 
 
 static void
-dump_data(std::ostream& out, ImageInput* input, const print_info_options& opt)
+dump_data(std::ostream& out, ImageInput* input, const print_info_options& opt,
+          int subimage)
 {
     using Strutil::print;
-    const ImageSpec& spec(input->spec());
+    ImageSpec spec = input->spec(subimage);
     if (spec.deep) {
         // Special handling of deep data
         DeepData dd;
-        if (!input->read_native_deep_image(dd)) {
+        if (!input->read_native_deep_image(subimage, 0, dd)) {
             print(out, "    dump data: could not read image\n");
             return;
         }
@@ -223,7 +226,7 @@ dump_data(std::ostream& out, ImageInput* input, const print_info_options& opt)
                         if (s)
                             print(out, " / ");
                         for (int c = 0; c < nc; ++c) {
-                            print(out, " {}=", spec.channelnames[c]);
+                            print(out, " {}=", spec.channel_name(c));
                             if (dd.channeltype(c) == TypeDesc::UINT)
                                 print(out, "{}",
                                       dd.deep_value_uint(pixel, c, s));
@@ -239,7 +242,7 @@ dump_data(std::ostream& out, ImageInput* input, const print_info_options& opt)
     } else {
         OIIO_UNUSED_OK bool ok = true;
         OIIO_DISPATCH_TYPES(ok, "dump_flat_data", dump_flat_data, spec.format,
-                            out, input, opt);
+                            out, input, opt, subimage);
     }
 }
 
@@ -609,7 +612,7 @@ print_info_subimage(std::ostream& out, Oiiotool& ot, int current_subimage,
         std::string sha = compute_sha1(ot, input, current_subimage);
         if (serformat == ImageSpec::SerialText)
             lines.insert(lines.begin() + 1, format("    SHA-1: {}", sha));
-        else if (serformat == ImageSpec::SerialText)
+        else if (serformat == ImageSpec::SerialXML)
             lines.insert(lines.begin() + 1, format("<SHA1>{}</SHA1>", sha));
     }
 
@@ -740,7 +743,7 @@ print_info_subimage(std::ostream& out, Oiiotool& ot, int current_subimage,
     if (input && opt.dumpdata) {
         ImageSpec tmp;
         input->seek_subimage(current_subimage, 0, tmp);
-        dump_data(out, input, opt);
+        dump_data(out, input, opt, current_subimage);
     }
 
     if (opt.compute_stats
